
' Definir una clase CommException que herede de la clase ApplicationException 
' e iniciar un objeto de tipo CommException al recibir un mensaje de error. 
Class CommException
    Inherits ApplicationException
    Sub New(ByVal Reason As String)
        MyBase.New(Reason)
    End Sub
End Class

Module modSerialPort
    'Declarar estructuras. 
    Public Structure DCB
        Public DCBlength As Int32
        Public BaudRate As Int32
        Public fBitFields As Int32
        'Ver los comentarios de Win32API.Txt 
        Public wReserved As Int16
        Public XonLim As Int16
        Public XoffLim As Int16
        Public ByteSize As Byte
        Public Parity As Byte
        Public StopBits As Byte
        Public XonChar As Byte
        Public XoffChar As Byte
        Public ErrorChar As Byte
        Public EofChar As Byte
        Public EvtChar As Byte
        Public wReserved1 As Int16 'Reservado; no usar 
    End Structure

    Public Structure COMMTIMEOUTS
        Public ReadIntervalTimeout As Int32
        Public ReadTotalTimeoutMultiplier As Int32
        Public ReadTotalTimeoutConstant As Int32
        Public WriteTotalTimeoutMultiplier As Int32
        Public WriteTotalTimeoutConstant As Int32
    End Structure

    'Declarar constantes. 
    Public Const GENERIC_READ As Int32 = &H80000000
    Public Const GENERIC_WRITE As Int32 = &H40000000
    Public Const OPEN_EXISTING As Int32 = 3
    Public Const FILE_ATTRIBUTE_NORMAL As Int32 = &H80
    Public Const NOPARITY As Int32 = 0
    Public Const ONESTOPBIT As Int32 = 0

    'Declarar referencias a funciones externas. 
    Public Declare Function CreateFile Lib "coredll.dll" _
        (ByVal lpFileName As String, ByVal dwDesiredAccess As Int32, _
         ByVal dwShareMode As Int32, ByVal lpSecurityAttributes As IntPtr, _
         ByVal dwCreationDisposition As Int32, ByVal dwFlagsAndAttributes As Int32, _
         ByVal hTemplateFile As IntPtr) As IntPtr

    Public Declare Function GetCommState Lib "coredll.dll" (ByVal nCid As IntPtr, _
        ByRef lpDCB As DCB) As Boolean

    Public Declare Function SetCommState Lib "coredll.dll" (ByVal nCid As IntPtr, _
        ByRef lpDCB As DCB) As Boolean

    Public Declare Function GetCommTimeouts Lib "coredll.dll" (ByVal hFile As IntPtr, _
        ByRef lpCommTimeouts As COMMTIMEOUTS) As Boolean

    Public Declare Function SetCommTimeouts Lib "coredll.dll" (ByVal hFile As IntPtr, _
        ByRef lpCommTimeouts As COMMTIMEOUTS) As Boolean

    Public Declare Function WriteFile Lib "coredll.dll" (ByVal hFile As IntPtr, _
        ByVal lpBuffer As Byte(), ByVal nNumberOfBytesToWrite As Int32, _
        ByRef lpNumberOfBytesWritten As Int32, ByVal lpOverlapped As IntPtr) As Boolean

    Public Declare Function ReadFile Lib "coredll.dll" (ByVal hFile As IntPtr, _
        ByVal lpBuffer As Byte(), ByVal nNumberOfBytesToRead As Int32, _
        ByRef lpNumberOfBytesRead As Int32, ByVal lpOverlapped As IntPtr) As Boolean

    Public Declare Function CloseHandle Lib "coredll.dll" (ByVal hObject As IntPtr) As Boolean

    Private hSerialPort As IntPtr

    ' Declarar las variables que utilizar para codificar. 
    Public oEncoder As New System.Text.ASCIIEncoding
    Public oEnc As System.Text.Encoding = oEncoder.GetEncoding(1252)
    Public Sub WritePort(ByRef Buffer() As Byte, ByVal BytesToWrite As Integer)
        Dim Success As Boolean
        Dim BytesWritten As Int32
        Try
            ' Escribir datos en COM1. 
            Success = WriteFile(hSerialPort, Buffer, BytesToWrite, BytesWritten, IntPtr.Zero)
            If Success = False Then
                Throw New CommException("No se puede escribir en COM1")
            End If
        Catch ex As Exception
            Console.WriteLine(ex.Message)
        End Try
    End Sub

    Public Sub ClosePort()
        Dim Success As Boolean
        Try
            ' Liberar el identificador para COM1. 
            Success = CloseHandle(hSerialPort)
            If Success = False Then
                Console.WriteLine("No se puede liberar identificador para COM1")
            End If
        Catch ex As Exception
            Console.WriteLine(ex.Message)
        End Try
    End Sub

    Public Function OpenPort(ByVal strCOM As String) As Integer
        Dim Success As Boolean
        Dim MyDCB As DCB
        Dim MyCommTimeouts As COMMTIMEOUTS
        Try
            ' Obtener un identificador para el puerto serie COM1. 
            hSerialPort = CreateFile(strCOM, GENERIC_READ Or GENERIC_WRITE, 0, IntPtr.Zero, _
                        OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, IntPtr.Zero)
            ' Comprobar que el identificador obtenido es vlido. 
            If hSerialPort.ToInt32 = -1 Then
                Throw New CommException("No se puede obtener un identificador para el puerto " & strCOM)
            End If
            ' Recuperar la configuracin actual del control. 
            Success = GetCommState(hSerialPort, MyDCB)
            If Success = False Then
                Throw New CommException("No se puede recuperar la configuracin actual del control")
            End If
            ' Modificar las propiedades de la estructura DCB recuperada de forma adecuada. 
            ' ADVERTENCIA: asegrese de modificar las propiedades segn sus valores compatibles. 
            MyDCB.BaudRate = 9600
            MyDCB.ByteSize = 8
            MyDCB.Parity = NOPARITY
            MyDCB.StopBits = ONESTOPBIT
            ' Reconfigurar COM1 segn las propiedades de la estructura DCB modificada. 
            Success = SetCommState(hSerialPort, MyDCB)
            If Success = False Then
                Throw New CommException("No se puede reconfigurar COM1")
            End If
            ' Recuperar la configuracin actual de tiempo de espera. 
            Success = GetCommTimeouts(hSerialPort, MyCommTimeouts)
            If Success = False Then
                Throw New CommException("No se puede recuperar la configuracin actual de tiempo de espera")
            End If
            ' Modificar las propiedades de la estructura COMMTIMEOUTS recuperada de forma adecuada. 
            ' ADVERTENCIA: asegrese de modificar las propiedades segn sus valores compatibles. 
            MyCommTimeouts.ReadIntervalTimeout = 0
            MyCommTimeouts.ReadTotalTimeoutConstant = 0
            MyCommTimeouts.ReadTotalTimeoutMultiplier = 0
            MyCommTimeouts.WriteTotalTimeoutConstant = 0
            MyCommTimeouts.WriteTotalTimeoutMultiplier = 0
            ' Reconfigurar el tiempo de espera segn las propiedades de la estructura COMMTIMEOUTS modificada. 
            Success = SetCommTimeouts(hSerialPort, MyCommTimeouts)
            If Success = False Then
                Throw New CommException("No se puede reconfigurar el tiempo de espera")
            End If
            Return 1
        Catch ex As Exception
            Console.WriteLine(ex.Message)
            Return 0
        End Try
    End Function

    Public Function ReadPort(ByVal BytesToRead As Integer, ByRef Buffer() As Byte) As Int32
        ' Declarar las variables local que utilizar en el cdigo. 
        Dim Success As Boolean
        Dim BytesRead As Int32
        Try
            ' Leer datos de COM1. 
            Success = ReadFile(hSerialPort, Buffer, BytesToRead, BytesRead, IntPtr.Zero)
            If Success = False Then
                Throw New CommException("No se puede leer de COM1")
            End If
            ReadPort = BytesRead
        Catch ex As Exception
            Console.WriteLine(ex.Message)
            ReadPort = -1
        End Try
    End Function


End Module
